require 'net/ssh'
require 'net/sftp'

class SSHDSL
  def initialize(conn, logger)
    @conn = conn
    @logger = logger
  end

  def exec(*command)
    cmd = command.flatten.join(' ')
    @logger.log(:ssh_exec, "Executing command #{cmd} on remote")
    @conn.exec!(cmd) do |channel, stream, data|
      {:stdout => :ssh_info, :stderr => :ssh_error}.each do |s, l|
        data.lines.each{ |d| @logger.log(l, d) if stream == s }
      end

      channel.on_request("exit-status") do |ch, data|
        exit_code = data.read_long
        raise "remote command: \"#{cmd}\" returned with non-zero exit code: #{exit_code}" if exit_code != 0
      end
    end
  end
end

class SFTPDSL
  def initialize(conn, logger)
    @conn = conn
    @logger = logger
  end

  def upload(from, to)
    Dir.glob(from).each do |src|
      to.flatten.each do |dst|
        dst += File.basename(src) if dst.end_with?('/')
        @logger.log(:sftp, "Uploading #{src} to #{dst}")
        # Maybe parallel uploads with @conn.upload!
        @conn.upload!(src, dst)
      end
    end
  end
end

class PlacerDSL
  def initialize(config, logger)
    @logger = logger

    instance_eval config
  end

  def ssh(&block)
    Net::SSH.start(@ssh_host, @ssh_user, @ssh_options) do |c|
      block.call SSHDSL.new(c, @logger)
    end
  end

  def sftp(&block)
    Net::SFTP.start(@ssh_host, @ssh_user, @ssh_options) do |c|
      block.call SFTPDSL.new(c, @logger)
    end
  end
end